// Copyright (C) 2003,2004,2005 by Object Mentor, Inc. All rights reserved.
// Released under the terms of the GNU General Public License version 2 or later.
package fitnesse.responders;

import fitnesse.Responder;
import fitnesse.FitNesseContext;
import fitnesse.wiki.*;
import fitnesse.util.XmlUtil;
import fitnesse.http.*;
import org.w3c.dom.*;

import java.text.SimpleDateFormat;
import java.util.Date;

public class RssResponderTest extends ResponderTest
{
    protected Element channelElement;
    protected Element rssElement;
    protected Document rssDoc;
    private String date;
    private String rfcDate;

    // Return an instance of the Responder being tested.
    protected Responder responderInstance()
    {
        return new RssResponder();
    }

    public void setUp() throws Exception
    {
        super.setUp();
        SimpleDateFormat dateFormat = new SimpleDateFormat(FitNesseContext.recentChangesDateFormat);
        date = dateFormat.format(new Date());
        SimpleDateFormat rfcDateFormat = new SimpleDateFormat(FitNesseContext.rfcCompliantDateFormat);
        rfcDate = rfcDateFormat.format(new Date());
    }

    public void testEmptyRssReport() throws Exception
    {
        buildRssChannel();
        assertEquals("rss", rssElement.getTagName());
        assertEquals("2.0", rssElement.getAttribute("version"));
        assertNotNull(channelElement);
        assertEquals("FitNesse:", XmlUtil.getTextValue(channelElement, "title"));
    }

    public void testOneNewPage() throws Exception
    {
        NodeList items = getReportedItems("|MyNewPage|me|" + date + "|");
        assertEquals(1, items.getLength());
        String title = "MyNewPage";
        String author = "me";
        String pubDate = rfcDate;
        String description = "me:" + rfcDate;
        checkItem(items.item(0), title, author, pubDate, description, "http://localhost/MyNewPage");
    }

    public void testTwoNewPages() throws Exception
    {
        String recentChangeOne = "|MyNewPage|me|" + date + "|";
        String recentChangeTwo = "|SomeOtherPage||" + date + "|";
        String recentChangesContent = recentChangeOne + "\n" + recentChangeTwo + "\n";
        NodeList items = getReportedItems(recentChangesContent);
        assertEquals(2, items.getLength());
        checkItem(items.item(0), "MyNewPage", "me", rfcDate, "me:" + rfcDate, "http://localhost/MyNewPage");
        checkItem(items.item(1), "SomeOtherPage", null, rfcDate, rfcDate, "http://localhost/SomeOtherPage");
    }

    public void testReportedPagesSelectedByResource() throws Exception
    {
        request.setResource("FrontPage");
        String page1 = "|SomePage|me|" + date + "|";
        String page2 = "|FrontPage|me|" + date + "|";
        String page3 = "|FrontPage.MyPage|me|" + date + "|";
        String page4 = "|SomePage.FrontPage|me|" + date;

        String recentChangesContent = page1 + "\n" + page2 + "\n" + page3 + "\n" + page4 + "\n";
        NodeList items = getReportedItems(recentChangesContent);
        assertEquals(2, items.getLength());
        checkItem(items.item(0), "FrontPage", "me", rfcDate, "me:" + rfcDate, "http://localhost/FrontPage");
        checkItem(items.item(1), "FrontPage.MyPage", "me", rfcDate, "me:" + rfcDate, "http://localhost/FrontPage.MyPage");
    }

    public void testLinkWithSetPrefix() throws Exception
    {
        PageData data = root.getData();
        data.setContent("!define RSS_PREFIX {http://host/}\n");
        root.commit(data);

        NodeList items = getReportedItems("|PageName|author|" + date + "|");
        assertEquals(1, items.getLength());
        checkItem(items.item(0), "PageName", "author", rfcDate, "author:" + rfcDate, "http://host/PageName");
    }

    public void testLinkWitDefaultPrefix() throws Exception
    {
        NodeList items = getReportedItems("|PageName|author|" + date + "|");
        assertEquals(1, items.getLength());
        checkItem(items.item(0), "PageName", "author", rfcDate, "author:"+rfcDate, "http://localhost/PageName");
    }

    public void testConvertDateFormat() throws Exception
    {
        SimpleDateFormat oldFormat = new SimpleDateFormat(FitNesseContext.recentChangesDateFormat);
        SimpleDateFormat newFormat = new SimpleDateFormat(FitNesseContext.rfcCompliantDateFormat);
        String inDate = oldFormat.format(new Date());
        String outDate = newFormat.format(new Date());
        RssResponder responder = new RssResponder();
        String convertedDate = responder.convertDateFormat(inDate);
        assertEquals(convertedDate, outDate);
    }

    public void testBadDateFormat() throws Exception
    {
        SimpleDateFormat oldFormat = new SimpleDateFormat("h:mm:ss a EEE MMM dd, yyyy");
        String inDate = oldFormat.format(new Date());
        RssResponder responder = new RssResponder();
        String convertedDate = responder.convertDateFormat(inDate);
        assertEquals(convertedDate, inDate);
    }

    private void buildRssChannel() throws Exception
    {
        SimpleResponse response = (SimpleResponse) responder.makeResponse(context, request);
        rssDoc = XmlUtil.newDocument(response.getContent());
        rssElement = rssDoc.getDocumentElement();
        channelElement = XmlUtil.getElementByTagName(rssElement, "channel");
    }

    private void checkItem(Node node, String title, String author, String pubDate, String description, String link) throws Exception
    {
        Element itemElement = (Element) node;
        assertEquals(title, XmlUtil.getTextValue(itemElement, "title"));
        assertEquals(author, XmlUtil.getTextValue(itemElement, "author"));
        assertEquals(pubDate, XmlUtil.getTextValue(itemElement, "pubDate"));
        assertEquals(description, XmlUtil.getTextValue(itemElement, "description"));
        assertEquals(link, XmlUtil.getTextValue(itemElement, "link"));
    }

    private NodeList getReportedItems(String recentChangesContent) throws Exception
    {
        crawler.addPage(root, PathParser.parse("RecentChanges"), recentChangesContent);
        buildRssChannel();
        NodeList items = channelElement.getElementsByTagName("item");
        return items;
    }
}
